﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.IO;
using System.Linq;
using System.Text;
using dbfs;
using Xunit;

public class ListFilesTests
{
	[Fact]
	public void FilesInRootFolderAfterCreatingNewBranch_Issue58()
	{
		Utils.ClearDB();

		using (DBFS dbfs = Utils.CreateDBFS())
		{
			byte[] buffer = Encoding.UTF8.GetBytes("this is content of a file");

			// 1. Upload any number of files and folders to branch Start.
			using (var commit = dbfs.StartCommit("", Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH)) {
				commit.AddFile("/test.file", p => new MemoryStream(buffer));
				commit.AddFile("/folder/test1.file", p => new MemoryStream(buffer));

				var result = commit.Finish();

				Assert.Equal(4, result.Changes.Count);
			}

			// 2. Create new branch Test.
			string new_branch = "Test";
			var branch = dbfs.CreateBranch(new_branch, Utils.DEFAULT_USER, Utils.DEFAULT_BRANCH);
			Assert.NotNull(branch);

			int latest_commit_id;

			// 3. Upload any file to non-root folder in branch Test.
			using (var commit = dbfs.StartCommit("", Utils.DEFAULT_USER, new_branch)) {
				commit.AddFile("/folder/test2.file", p => new MemoryStream(buffer));

				var result = commit.Finish();

				Assert.Equal(1, result.Files.Count);

				latest_commit_id = result.Id;
			}

			// 4. Root folder appears to be empty in branch Test (and OK in branch Start).
			{
				var files = dbfs.GetFilesList(latest_commit_id, "/").ToList();

				Assert.Equal(1, files.Count);
				Assert.Equal("/test.file", files[0].Path);

				files = dbfs.GetFilesList(branch.Name, "/").ToList();

				Assert.Equal(1, files.Count);
				Assert.Equal("/test.file", files[0].Path);
			}
			{
				var folders = dbfs.GetFoldersList(latest_commit_id, "/").ToList();

				Assert.Equal(1, folders.Count);
				Assert.Equal("/folder/", folders[0].Path);

				folders = dbfs.GetFoldersList(branch.Name, "/").ToList();

				Assert.Equal(1, folders.Count);
				Assert.Equal("/folder/", folders[0].Path);
			}
		}
	}
}
