﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace dbfs
{
	internal static partial class Helpers
	{
		public static void ThrowIfNull<T>(this T argument, string name)
		{
			if (argument == null)
				throw new ArgumentNullException(name);
		}

		public static void ThrowIfNullOrEmptry(this string argument, string name)
		{
			if (String.IsNullOrEmpty(argument))
			{
				throw new ArgumentNullException(name);
			}
		}

		public static R _<T, R>(this T self, Func<T, R> fn, R @null = default(R))
		{
			if (self == null) return @null;

			return fn(self);
		}

		#region IDictionary helpers

		public static int RemoveRange<T, P, R>(this IDictionary<T, P> target, IEnumerable<R> range, Func<R, T> key_selector)
		{
			int result = 0;

			foreach (var e in range) {
				if (target.Remove(key_selector(e)))
					++result;
			}

			return result;
		}

		#endregion

		public static IList<string> AffectedFolders(IList<string> added_files)
		{
			return added_files.SelectMany(e => GetAllFoldersFromSinglePath(e)).Distinct().ToList();
		}

		/// <summary>
		/// Ignores name after last '/'. E.g.: /some/path/to/file/ignored_name
		/// </summary>
		public static IList<string> GetAllFoldersFromSinglePath(string path)
		{
			string[] list = path.Split('/');

			var result = new List<string>(list.Length - 1);
			result.Add("/");

			for (int i = 1; i < list.Length - 1; ++i) {
				result.Add(result[i - 1] + list[i] + '/');
			}

			return result;
		}

		public static string MovePath(this string path, string from, string to)
		{
			if (!path.StartsWith(from)) throw new ArgumentException("Path " + path + " doesn't starts from " + from);

			return to + path.Substring(from.Length);
		}


		public static string EnsureFile(ref string path)
		{
			if (path.IsFolder()) path = path.Substring(0, path.Length - 1);

			return path;
		}

		public static string EnsureFolder(ref string path)
		{
			if (!path.IsFolder()) path += '/';

			return path;
		}

		public static string EnsureFolder(this string path)
		{
			return path.IsFolder() ? path : path + '/';
		}

		public static bool IsFolder(this string path)
		{
			return path.EndsWith("/");
		}
	}
}
