﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.repository
{
	using System.IO;
	using System.Linq;
	using dbfs.model;
	using dbfs.utils;
	using dbfs.utils.db;

	class BlobManagement : RepositoryManagement
	{
		const string SCHEMA = "dbfs";
		const string TABLE = "Blobs";
		const string CONTENT_FIELD = "content";
		const string KEY_FIELD = "id";
		const string HASH_FIELD = "sha1";

		public Blob TryGet(int id)
		{
			return _context.Blobs.FirstOrDefault(e => e.id == id);
		}

		public Blob TryGet(string sha1)
		{
			return _context.Blobs.Where(e => e.sha1 == sha1).FirstOrDefault();
		}

		public Stream GetContent(int id)
		{
			return _context.Database.Connection.GetFieldReaderStream(SCHEMA, TABLE, CONTENT_FIELD, KEY_FIELD, id);
		}

		public Stream GetContent(string sha1)
		{
			return _context.Database.Connection
				.GetFieldReaderStream(SCHEMA, TABLE, CONTENT_FIELD, HASH_FIELD, sha1);
		}

		public Blob Save(Stream content)
		{
			var blob = new Blob();

			_context.Blobs.Add(blob); // 12ms average
			Repository.SaveChanges();

			long size;

			blob.sha1 = CryptUtils.ComputeHashWhile(content, (stream) =>
				{
					_context.Database.Connection
						.SetFieldFromStream(stream, SCHEMA, TABLE, CONTENT_FIELD, KEY_FIELD, blob.id);
				},
				out size
			);
			blob.size = size;

			Repository.SaveChanges();

			return blob;
		}
	}
}
