﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.repository
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using dbfs.model;
	using dbfs.exceptions;
	using dbfs.utils;

	class CommitManagement : RepositoryManagement
	{
		#region create

		public Commit Create(string message, User user)
		{
			return new Commit {
				message = message,
				User = user,
				date = DateTime.UtcNow
			};
		}

		public BranchCommit CreateRelation(Branch branch, Commit commit)
		{
			// todo: 40ms?
			return _context.BranchCommits.Add(new BranchCommit {
				Branch = branch,
				Commit = commit
			});
		}

		#endregion

		#region getters

		public Commit TryGet(int id)
		{
			return _context.Commits.FirstOrDefault(c => c.id == id);
		}

		/// <exception cref="CommitDoesNotExistException"></exception>
		public Commit Get(int id)
		{
			var commit = TryGet(id);

			if (commit == null)
				throw new CommitDoesNotExistException(id);

			return commit;
		}

		#endregion

		#region count

		public int Count(int branch_id)
		{
			return _context.BranchCommits
				.Where(e => e.branch_id == branch_id)
				.Count()
			;
		}

		public int Count(int branch_id, string path)
		{
			return _context.BranchCommits
				.Where(e => e.branch_id == branch_id)
				.Select(e => e.Commit)
				.Join(_context.FileRevisions.Where(e => e.File.path == path),
					e => e.id, e => e.commit_id, (c, r) => c)
				.Count()
			;
		}

		#endregion

		#region lists

		public List<Commit> List(int branch_id, int start = 0, int max_count = 0)
		{
			return _context.BranchCommits
				.Where(e => e.branch_id == branch_id)
				.Select(e => e.Commit)
				.If(start != 0, q => q.Skip(start))
				.If(max_count != 0, q => q.Take(max_count))
				.ToList()
			;
		}

		public List<Commit> List(string path, int branch_id, int start = 0, int max_count = 0)
		{
			return _context.BranchCommits
				.Where(e => e.branch_id == branch_id)
				.Select(e => e.Commit)
				.Join(_context.FileRevisions.Where(e => e.File.path == path),
					e => e.id, e => e.commit_id, (c, r) => c)
				.If(start != 0, e => e.Skip(start))
				.If(max_count != 0, e => e.Take(max_count))
				.ToList()
			;
		}

		#endregion
	}
}
