/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.dbfscmd
{
	using System;
	using ManyConsole;

	public class BranchCommand : ConsoleCommand
	{
		public BranchCommand()
		{
			this.SkipsCommandSummaryBeforeRunning();
			this.IsCommand("branch", "Operations with branches.");

			this.HasOption("list", "list branches", v => _list = v != null);
			this.HasOption<string>("user=", "provide user name", v => this._user = v);
			this.HasOption("remove", "remove specified branch", v => this._remove = v != null);

			this.AllowsAnyAdditionalArguments("<branch-name> <start-point>");
		}

		private bool _list;
		private string _user;
		private bool _remove;

		public override int Run(string[] remainingArguments)
		{
			this._checkArguments(remainingArguments);

			var dbfs = Program.GetDBFS();

			if (this._list || remainingArguments.Length == 0)
			{
				foreach (var branch in dbfs.ListBranches())
				{
					this._print(branch);
				}
			}
			else if (this._remove) {
				string branch_name = remainingArguments[0];

				dbfs.RemoveBranch(branch_name);

				Console.WriteLine("Branch {0} has been removed", branch_name);
			}
			else {
				string branch_name = remainingArguments[0];
				string parent_name = remainingArguments[1];

				this._print(dbfs.CreateBranch(branch_name, this._user, parent_name));
			}

			return 0;
		}

		private void _checkArguments(string[] remainingArguments)
		{
			int count = 0;

			if (this._list) ++count;
			if (!String.IsNullOrEmpty(this._user)) ++count;
			if (this._remove) ++count;

			if (count > 1)
				throw new ConsoleHelpAsException("only single command (--user, --list and --remove) allowed at the same time");
		}

		private void _print(dbfs.Branch branch)
		{
			Console.WriteLine(
				"{0,-20} # rev.{1,-4} {2,-20} # created by {3}",
				branch.Name, branch.LatestCommit.Id,
				branch.LatestCommit.UserName,
				branch.UserName
			);
		}
	}
}
