﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.dbfscmd
{
	using System;
	using System.Collections.Generic;
	using System.Linq;
	using ManyConsole;

	public class LogCommand : ConsoleCommand
	{
		public LogCommand()
		{
			this.SkipsCommandSummaryBeforeRunning();
			this.IsCommand("log");

			this.HasRequiredOption<string>("b|branch=", "provide branch name", v => m_branch = v);
			this.HasOption<int>("max-count=", "limit output to N latest commits", v => m_max_count = v);
			this.HasOption<int>("start=", "only for 'log for all'", v => m_start = v);

			this.AllowsAnyAdditionalArguments("[virtual-path]");
		}

		private string m_branch;
		private int m_max_count = 0;

		private int m_start = 0;

		public override int Run(string[] remainingArguments)
		{
			var dbfs = Program.GetDBFS();
			string path = remainingArguments.Length >= 1 ? remainingArguments[0] : null;

			IEnumerable<dbfs.Commit> log;

			if (String.IsNullOrEmpty(path)) {
				log = dbfs.GetLog(m_branch, m_start, m_max_count);
			}
			else {
				if (String.IsNullOrEmpty(m_branch))
					throw new ConsoleHelpAsException("branch must be known for certain path");

				log = path.EndsWith("/")
					? dbfs.GetLogForFolder(path, m_branch, m_max_count)
					: dbfs.GetLogForFile(path, m_branch, m_max_count);
			}

			m_printLog(log.ToList());

			return 0;
		}

		void m_printLog(List<dbfs.Commit> log)
		{
			for (int i = 0; i < log.Count; ++i) {
				var log_item = log[i];

				if (i != 0) Console.WriteLine();

				Console.WriteLine("commit {0}\nAuthor: {1}\nDate:   {2}\n\n    {3}",
					log_item.Id, log_item.UserName, log_item.Date, log_item.Message);
			}
		}
	}
}
