﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Xml;

namespace dbfs.utils
{
	public class Config
	{
		public Config(string filename, bool ignoreFileError = false,string rootname="config" )
		{
			_doc = new XmlDocument();
            _filename = filename;
			try
			{
				_doc.Load(filename);
			}
			catch (System.IO.IOException)
			{
				if (!ignoreFileError)
					throw;
			}

			_root = _doc.SelectSingleNode("/config") as XmlElement;
		}

		public bool IsValid { get { return _root != null; } }

		public string GetText(string xpath)
		{
			XmlNode node = IsValid ? _root.SelectSingleNode(xpath) : null;
			return node != null ? node.InnerText : "";
        }
        public void WriteText(string xpath,string value)
        {
            XmlNode node = IsValid ? _root.SelectSingleNode(xpath) : null;
            if (node != null)
            {
                node.InnerText = value;
                _doc.Save(_filename);
            }
        }
		public string GetAttribute(string xpath, string attr)
		{
			XmlElement el = IsValid ? _root.SelectSingleNode(xpath) as XmlElement : null;
			return el != null ? el.GetAttribute(attr) : "";
		}

		public string[] GetList(string xpath)
		{
			return IsValid
				? _root.SelectNodes(xpath).Cast<XmlElement>().Select(e => e.InnerText).ToArray()
				: new string[0];
		}

		public static Config Get(string name, bool ignoreFileError = false)
		{
			Config instance = null;

			_lock.AcquireReaderLock(Timeout.Infinite);
			_instances.TryGetValue(name, out instance);
			_lock.ReleaseReaderLock();

			if (instance != null && (ignoreFileError || instance.IsValid))
				return instance;

			string filename = name.Contains('.') ? name : name + ".config";

			instance = new Config(FileUtils.GetFullPath(filename), ignoreFileError);

			Add(name, instance);

			return instance;
		}

		public static void Add(string name, Config instance)
		{
			_lock.AcquireWriterLock(Timeout.Infinite);
			_instances[name] = instance;
			_lock.ReleaseWriterLock();
		}

		private XmlDocument _doc;
        private string _filename;
		private XmlElement _root;

		private static Dictionary<string, Config> _instances = new Dictionary<string, Config>();
		private static readonly ReaderWriterLock _lock = new ReaderWriterLock();
	}
}
