﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;

namespace dbfs.utils
{
	public static class CryptUtils
	{
		public static string ComputeHash(string text)
		{
			using (var sha = new SHA1Managed())
			{
				return ByteArrayToHexString(sha.ComputeHash(StringToByteArray(text)));
			}
		}

		public static string ComputeHash(Stream stream)
		{
			using (var sha = new SHA1Managed())
			{
				long pos = stream.CanSeek ? stream.Position : 0;

				byte[] hash = sha.ComputeHash(stream);

				if (stream.CanSeek)
					stream.Position = pos;

				return ByteArrayToHexString(hash);
			}
		}

		public static byte[] StringToByteArray(string text)
		{
			return Encoding.UTF8.GetBytes(text);
		}

		public static string ByteArrayToHexString(byte[] array)
		{
			return BitConverter.ToString(array).Replace("-", "").ToLower();
		}

		public static string StringToHexString(string text)
		{
			return ByteArrayToHexString(StringToByteArray(text));
		}

		public static string ComputeHashWhile(Stream stream, Action<Stream> while_)
		{
			long size;

			return ComputeHashWhile(stream, while_, out size);
		}

		public static string ComputeHashWhile(Stream stream, Action<Stream> while_, out long size)
		{
			using (var crypto = new SHA1Managed())
			{
				long pos = stream.CanSeek ? stream.Position : 0;
				long total_size = 0;

				var wrapped = new StreamWithEvents(stream,
					(buffer, offset, count) =>
					{
						total_size += count;
						crypto.TransformBlock(buffer, offset, count, null, 0);
					},
					null
				);

				while_.Invoke(wrapped);

				crypto.TransformFinalBlock(new byte[0], 0, 0);

				if (stream.CanSeek)
					stream.Position = pos;

				size = total_size;

				return ByteArrayToHexString(crypto.Hash);
			}
		}
	}
}
