﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Reflection;

namespace dbfs.utils
{
	public static class ExceptionUtils
	{
		public static string ExtractMessage(this Exception e, bool systemStacktrace = false)
		{
			Exception outer = e;

			while (outer is TargetInvocationException && outer.InnerException != null)
				outer = outer.InnerException;

			string message = Strip(outer.Message);
			string shift = "  ";

			for (Exception inner = outer.InnerException; inner != null; inner = inner.InnerException)
			{
				if (inner is TargetInvocationException)
					continue;

				message = string.Format(
					"{0}\n{1}{2}",
					message, shift,
					Strip(inner.Message).Replace("\n", "\n" + shift)
				);

				if (systemStacktrace && IsSystem(inner) && !IsSystem(outer) && inner.StackTrace != null)
				{
					message += string.Format(
						"\n\n{0}Stack trace:\n{0}{1}",
						shift, inner.StackTrace.Replace("\n", "\n" + shift)
					);
				}

				shift += "  ";
				outer = inner;
			}

			if (systemStacktrace && IsSystem(e) && e.StackTrace != null)
			{
				message += "\n\nStack trace:\n" + e.StackTrace;
			}

			return message;
		}

		private static bool IsSystem(Exception e)
		{
			return e.GetType().FullName.StartsWith("System.");
		}

		private static string Strip(string message)
		{
			string strip = message
				.Replace(" See the inner exception for details.", "")
				.Replace(" See the InnerException for details.", "")
				.TrimEnd();
			
			if (strip.IndexOf('\n') < 0)
				return strip.TrimEnd('.') + ".";

			return strip;
		}
	}
}
