﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace dbfs.utils
{
	public static class ObjectUtils
	{
		public static string Dump(object obj)
		{
			if (obj == null)
				return "null";

			if (obj is bool)
				return (bool)obj ? "true" : "false";

			if (obj is string)
				return "'" + ((string)obj).Replace("'", "\\'") + "'";

			return obj.ToString();
		}

		public static IDictionary<string, object> AsDictionary(object obj)
		{
			if (obj == null)
				return null;

			if (obj is IDictionary<string, object>)
				return (IDictionary<string, object>)obj;

			if (obj is IDictionary)
			{
				Dictionary<string, object> dict = new Dictionary<string, object>();

				foreach (DictionaryEntry pair in (IDictionary)obj)
					dict[pair.Key.ToString()] = pair.Value;

				return dict;
			}

			var objDict = obj.GetType().GetFields()
				.ToDictionary(f => f.Name, f => f.GetValue(obj));

			foreach (var prop in obj.GetType().GetProperties())
			{
				objDict[prop.Name] = prop.GetValue(obj, null);
			}

			return objDict;
		}

		/// <summary>
		/// Executes @true if condition == true and returns result.
		/// </summary>
		public static T If<T>(this T self, bool condition, Func<T, T> @true)
		{
			return condition ? @true(self) : self;
		}

		/// <summary>
		/// Executes @true or @false (depends of condition value) and returns result.
		/// </summary>
		public static R If<T, R>(this T self, bool condition, Func<T, R> @true, Func<T, R> @false)
		{
			return condition ? @true(self) : @false(self);
		}
	}
}
