﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Runtime.Serialization;

namespace dbfs.utils
{
	/// <summary>
	/// Represents a weak reference, which references an object while still allowing
	/// that object to be reclaimed by garbage collection.
	/// </summary>
	/// <typeparam name="T">The type of the object that is referenced.</typeparam>
	[Serializable]
	public class WeakReference<T> : WeakReference where T : class
	{
		/// <summary>
		/// Initializes a new instance of the WeakReference{T} class, referencing
		/// the specified object.
		/// </summary>
		/// <param name="target">The object to reference.</param>
		public WeakReference(T target) : base(target)
		{
		}

		/// <summary>
		/// Initializes a new instance of the WeakReference{T} class, referencing
		/// the specified object and using the specified resurrection tracking.
		/// </summary>
		/// <param name="target">An object to track.</param>
		/// <param name="trackResurrection">Indicates when to stop tracking the object.
		/// If true, the object is tracked after finalization;
		/// if false, the object is only tracked until finalization.
		/// </param>
		public WeakReference(T target, bool trackResurrection) : base(target, trackResurrection)
		{
		}

		protected WeakReference(SerializationInfo info, StreamingContext context)
		 : base(info, context)
		{
		}

		/// <summary>
		/// Gets or sets the object (the target) referenced by the 
		/// current WeakReference{T} object.
		/// </summary>
		public new T Target
		{
			get
			{
				return (T)base.Target;
			}
			set
			{
				base.Target = value;
			}
		}
	}
}
