﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.Data;
using System.Data.Common;
using System.Linq;

namespace dbfs.utils.db.impl.mssql
{
	class MsSqlBinaryFieldWriter : IBinaryFieldWriter
	{
		public MsSqlBinaryFieldWriter(DbConnection connection, string schema, string table, string write_field, string key_field, object key)
		{
			using (DbCommand cmd = connection.CreateCommand())
			{
				cmd.CommandType = CommandType.Text;
				cmd.CommandTimeout = 5 * 60;
				cmd.CommandText = string.Format(
					"UPDATE {0} SET [{1}] = @Bytes WHERE [{2}] = @KeyValue",
					Helpers.TableSource(schema, table), write_field, key_field
				);

				DbParameter keyValue = cmd.CreateParameter();
				keyValue.ParameterName = "@KeyValue";
				keyValue.Value = key;
				cmd.Parameters.Add(keyValue);

				DbParameter data = cmd.CreateParameter();
				data.ParameterName = "@Bytes";
				data.DbType = DbType.Binary;
				data.Size = 0;
				data.Value = new byte[0];
				cmd.Parameters.Add(data);

				cmd.ExecuteNonQuery();
			}

			_cmd = connection.CreateCommand();
			_cmd.CommandTimeout = 5 * 60;

			_cmd.CommandType = CommandType.Text;
			_cmd.CommandText = string.Format(
				"UPDATE {0} SET [{1}] .WRITE (@Bytes, @Offset, @Length) WHERE [{2}] = @KeyValue",
				Helpers.TableSource(schema, table), write_field, key_field
			);

			DbParameter keyParam = _cmd.CreateParameter();
			keyParam.ParameterName = "@KeyValue";
			keyParam.Value = key;
			_cmd.Parameters.Add(keyParam);

			_bytesParam = _cmd.CreateParameter();
			_bytesParam.ParameterName = "@Bytes";
			_bytesParam.DbType = DbType.Binary;
			_cmd.Parameters.Add(_bytesParam);

			_offsetParam = _cmd.CreateParameter();
			_offsetParam.ParameterName = "@Offset";
			_offsetParam.DbType = DbType.Double; //SqlDbType.Int
			_cmd.Parameters.Add(_offsetParam);

			_lengthParam = _cmd.CreateParameter();
			_lengthParam.ParameterName = "@Length";
			_lengthParam.DbType = DbType.Double; //SqlDbType.Int
			_cmd.Parameters.Add(_lengthParam);
		}

		public int Position { get; set; }

		public void Write(byte[] buffer, int offset, int count)
		{
			buffer = buffer.Skip(offset).Take(count).ToArray();

			_offsetParam.Value = Position;
			_lengthParam.Value = buffer.Length;	// overwrite count original bytes
			_bytesParam.Value = buffer;
			_bytesParam.Size = buffer.Length;

			_cmd.ExecuteNonQuery();

			Position += buffer.Length;
		}

		public void Dispose()
		{
			if (_cmd != null)
			{
				_bytesParam = null;
				_offsetParam = null;
				_lengthParam = null;

				_cmd.Dispose();
				_cmd = null;
			}
		}

		private DbCommand _cmd;
		private DbParameter _bytesParam;
		private DbParameter _offsetParam;
		private DbParameter _lengthParam;
	}
}
